#ifndef MSP_IO_BUFFERED_H_
#define MSP_IO_BUFFERED_H_

#include <sigc++/trackable.h>
#include "base.h"

namespace Msp {
namespace IO {

class Buffered: public Base, public sigc::trackable
{
private:
	Base &below;
	std::size_t buf_size = 0;
	char *buf = nullptr;
	char *begin = nullptr;
	char *end = nullptr;
	Mode cur_op = M_NONE;

public:
	Buffered(Base &, unsigned = 8192);
	~Buffered();

	void set_block(bool) override;
	void set_inherit(bool) override;

	void flush();

protected:
	std::size_t do_write(const char *, std::size_t) override;
	std::size_t do_read(char *, std::size_t) override;
public:
	std::size_t put(char) override;

	bool getline(std::string &);
	int get();

	const Handle &get_handle(Mode) override;

private:
	void set_op(Mode);
public:
	Mode get_current_op() const { return cur_op; }
	std::size_t get_current_size() const;
};

} // namespace IO
} // namespace Msp

#endif
