#ifndef MSP_IO_MEMORY_H_
#define MSP_IO_MEMORY_H_

#include "seekable.h"

namespace Msp {
namespace IO {

class Memory: public Seekable
{
private:
	char *begin = nullptr;
	char *end = nullptr;
	char *pos = nullptr;

public:
	Memory(char *d, std::size_t s, Mode m = M_RDWR): Memory(d, d+s, m) { }
	Memory(char *, char *, Mode = M_RDWR);
	Memory(const char *d, std::size_t s): Memory(const_cast<char *>(d), const_cast<char *>(d+s), M_READ) { }
	Memory(const char *b, const char *e): Memory(const_cast<char *>(b), const_cast<char *>(e), M_READ) { }

	void set_block(bool) override;
	void set_inherit(bool) override;

private:
	std::size_t do_write(const char *, std::size_t) override;
	std::size_t do_read(char *, std::size_t) override;
public:
	std::size_t put(char) override;
	bool getline(std::string &) override;
	int get() override;

	const Handle &get_handle(Mode) override;

	SeekOffset seek(SeekOffset, SeekType) override;
	SeekOffset tell() const override { return pos-begin; }
};

} // namespace IO
} // namespace Msp

#endif
